@@ -30,22 +30,25 @@ class AbstractQueryManager(object):
 
         Issue a warning if the state is not in the list of valid states.
 
-        :param string state: The state to be used to filter (should be a string among
+        :param state: The state to be used to filter (should be a string among
                 those defined in aiida.common.datastructures.calc_states)
+        :type state: str
         :param computer: a Django DbComputer entry, or a Computer object, of a
                 computer in the DbComputer table.
                 A string for the hostname is also valid.
         :param user: a Django entry (or its pk) of a user in the DbUser table;
                 if present, the results are restricted to calculations of that
                 specific user
-        :param bool only_computer_user_pairs: if False (default) return a queryset
+        :param only_computer_user_pairs: if False (default) return a queryset
                 where each element is a suitable instance of Node (it should
                 be an instance of Calculation, if everything goes right!)
                 If True, return only a list of tuples, where each tuple is
                 in the format
                 ('dbcomputer__id', 'user__id')
                 [where the IDs are the IDs of the respective tables]
-        :param int limit: Limit the number of rows returned
+        :type only_computer_user_pairs: bool
+        :param limit: Limit the number of rows returned
+        :type limit: int
 
         :return: a list of calculation objects matching the filters.
         """
@@ -111,15 +114,15 @@ class AbstractQueryManager(object):
 
     def get_creation_statistics(
             self,
-            user_email=None
+            user_pk=None
     ):
         """
         Return a dictionary with the statistics of node creation, summarized by day.
 
         :note: Days when no nodes were created are not present in the returned `ctime_by_day` dictionary.
 
-        :param user_email: If None (default), return statistics for all users.
-            If an email is specified, return only the statistics for the given user.
+        :param user_pk: If None (default), return statistics for all users.
+            If user pk is specified, return only the statistics for the given user.
 
         :return: a dictionary as
             follows::
@@ -178,8 +181,9 @@ class AbstractQueryManager(object):
 
         q = QB()
         q.append(Node, project=['id', 'ctime', 'type'], tag='node')
-        if user_email is not None:
-            q.append(User, creator_of='node', project='email', filters={'email': user_email})
+
+        if user_pk is not None:
+            q.append(User, creator_of='node', project='email', filters={'pk': user_pk})
         qb_res = q.all()
 
         # total count
