@@ -16,9 +16,202 @@ from sqlalchemy.exc import StatementError
 from aiida.backends.testbase import AiidaTestCase
 from aiida.common.exceptions import ModificationNotAllowed, UniquenessError
 from aiida.common.links import LinkType
+from aiida.common import caching
+from aiida.orm.calculation import Calculation
+from aiida.orm.code import Code
 from aiida.orm.data import Data
 from aiida.orm.node import Node
 from aiida.orm.utils import load_node
+from aiida.utils.capturing import Capturing
+from aiida.utils.delete_nodes import delete_nodes
+
+
+class TestNodeHashing(AiidaTestCase):
+    """
+    Tests the functionality of hashing a node
+    """
+
+    @staticmethod
+    def create_simple_node(a, b=0, c=0):
+        n = Node()
+        n._set_attr('a', a)
+        n._set_attr('b', b)
+        n._set_attr('c', c)
+        return n
+
+    def test_simple_equal_nodes(self):
+        attributes = [
+            (1.0, 1.1, 1.2),
+            ({'a': 'b', 'c': 'd'}, [1, 2, 3], {4, 1, 2})
+        ]
+        for attr in attributes:
+            n1 = self.create_simple_node(*attr)
+            n2 = self.create_simple_node(*attr)
+            n1.store(use_cache=True)
+            n2.store(use_cache=True)
+            self.assertEqual(n1.uuid, n2.get_extra('_aiida_cached_from'))
+
+
+    def test_node_uuid_hashing_for_querybuidler(self):
+        """
+        QueryBuilder results should be reusable and shouldn't brake hashing.
+        """
+        from aiida.orm.node import Node
+        from aiida.orm.querybuilder import QueryBuilder
+
+        n = Node()
+        n.store()
+
+        # Search for the UUID of the stored node
+        qb = QueryBuilder()
+        qb.append(Node, project=['uuid'],
+                  filters={'id': {'==': n.id}})
+        [uuid] = qb.first()
+
+        # Look the node with the previously returned UUID
+        qb = QueryBuilder()
+        qb.append(Node, project=['id'],
+                  filters={'uuid': {'==': uuid}})
+
+        # Check that the query doesn't fail
+        qb.all()
+        # And that the results are correct
+        self.assertEquals(qb.count(), 1)
+        self.assertEquals(qb.first()[0], n.id)
+
+    @staticmethod
+    def create_folderdata_with_empty_file():
+        from aiida.orm.data.folder import FolderData
+        res = FolderData()
+        with res.folder.get_subfolder('path').open('name', 'w') as f:
+            pass
+        return res
+
+    @staticmethod
+    def create_folderdata_with_empty_folder():
+        from aiida.orm.data.folder import FolderData
+        res = FolderData()
+        res.folder.get_subfolder('path/name').create()
+        return res
+
+    def test_folder_file_different(self):
+        f1 = self.create_folderdata_with_empty_file()
+        f2 = self.create_folderdata_with_empty_folder()
+
+        assert (
+                f1.folder.get_subfolder('path').get_content_list() ==
+                f2.folder.get_subfolder('path').get_content_list()
+        )
+        assert f1.get_hash() != f2.get_hash()
+
+    def test_folder_same(self):
+        f1 = self.create_folderdata_with_empty_folder()
+        f2 = self.create_folderdata_with_empty_folder()
+        f1.store()
+        f2.store(use_cache=True)
+        assert f1.uuid == f2.get_extra('_aiida_cached_from')
+
+    def test_file_same(self):
+        f1 = self.create_folderdata_with_empty_file()
+        f2 = self.create_folderdata_with_empty_file()
+        f1.store()
+        f2.store(use_cache=True)
+        assert f1.uuid == f2.get_extra('_aiida_cached_from')
+
+    def test_simple_unequal_nodes(self):
+        attributes = [
+            [(1.0, 1.1, 1.2), (2.0, 1.1, 1.2)],
+            [(1e-14,), (2e-14,)],
+        ]
+        for attr1, attr2 in attributes:
+            n1 = self.create_simple_node(*attr1)
+            n2 = self.create_simple_node(*attr2)
+            n1.store()
+            n2.store(use_cache=True)
+            self.assertNotEquals(n1.uuid, n2.uuid)
+            self.assertFalse('_aiida_cached_from' in n2.extras())
+
+    def test_unequal_arrays(self):
+        import numpy as np
+        from aiida.orm.data.array import ArrayData
+        arrays = [
+            (np.zeros(1001), np.zeros(1005)),
+            (np.array([1, 2, 3]), np.array([2, 3, 4]))
+        ]
+
+        def create_arraydata(arr):
+            a = ArrayData()
+            a.set_array('a', arr)
+            return a
+
+        for arr1, arr2 in arrays:
+            a1 = create_arraydata(arr1)
+            a1.store()
+            a2 = create_arraydata(arr2)
+            a2.store(use_cache=True)
+            self.assertNotEquals(a1.uuid, a2.uuid)
+            self.assertFalse('_aiida_cached_from' in a2.extras())
+
+    def test_updatable_attributes(self):
+        """
+        Tests that updatable attributes are ignored.
+        """
+        c = Calculation()
+        hash1 = c.get_hash()
+        c._set_attr('_sealed', True)
+        hash2 = c.get_hash()
+        self.assertNotEquals(hash1, None)
+        self.assertEquals(hash1, hash2)
+
+
+class TestDataNode(AiidaTestCase):
+    """
+    These tests check the features of Data nodes that differ from the base Node
+    """
+    boolval = True
+    intval = 123
+    floatval = 4.56
+    stringval = "aaaa"
+    # A recursive dictionary
+    dictval = {
+        'num': 3,
+        'something': 'else',
+        'emptydict': {},
+        'recursive': {
+            'a': 1,
+            'b': True,
+            'c': 1.2,
+            'd': [1, 2, None],
+            'e': {
+                'z': 'z',
+                'x': None,
+                'xx': {},
+                'yy': []
+            }
+        }
+    }
+    listval = [1, "s", True, None]
+    emptydict = {}
+    emptylist = []
+
+    def test_attr_after_storing(self):
+        a = Data()
+        a._set_attr('bool', self.boolval)
+        a._set_attr('integer', self.intval)
+        a.store()
+
+        # And now I try to edit/delete the keys; I should not be able to do it
+        # after saving. I try only for a couple of attributes
+        with self.assertRaises(ModificationNotAllowed):
+            a._del_attr('bool')
+        with self.assertRaises(ModificationNotAllowed):
+            a._set_attr('integer', 13)
+
+    def test_modify_attr_after_store(self):
+        a = Data()
+        a.store()
+        with self.assertRaises(ModificationNotAllowed):
+            a._set_attr('i', 12)
 
 
 class TestTransitiveNoLoops(AiidaTestCase):
@@ -328,7 +521,6 @@ class TestNodeBasic(AiidaTestCase):
         # Now I check if I can retrieve them, before the storage
         self.assertEquals(a.get_attrs(), target_attrs)
 
-
     def test_store_object(self):
         """Trying to store objects should fail"""
         a = Node()
@@ -364,7 +556,7 @@ class TestNodeBasic(AiidaTestCase):
         self.assertEquals(a.get_attr('list'), [1, 2, 3, 4])
         self.assertEquals(mylist, [1, 2, 3])
 
-    #pylint: disable=no-self-use,unused-argument,unused-variable,no-member
+    # pylint: disable=no-self-use,unused-argument,unused-variable,no-member
     def DISABLED(self):
         """
         This test routine is disabled for the time being; I will re-enable
@@ -400,7 +592,7 @@ class TestNodeBasic(AiidaTestCase):
 
             all_keys = models.DbAttribute.objects.filter(
                 dbnode=n.dbnode).values_list(
-                    'key', flat=True)
+                'key', flat=True)
 
             print max(len(i) for i in all_keys)
 
@@ -454,6 +646,7 @@ class TestNodeBasic(AiidaTestCase):
             'bool': 'some non-boolean value',
             'some_other_name': 987
         }
+        all_extras = dict(_aiida_hash=AnyValue(), **extras_to_set)
 
         for k, v in extras_to_set.iteritems():
             a.set_extra(k, v)
@@ -478,12 +671,12 @@ class TestNodeBasic(AiidaTestCase):
         b.store()
         # and I finally add a extras
         b.set_extra('meta', 'textofext')
-        b_expected_extras = {'meta': 'textofext'}
+        b_expected_extras = {'meta': 'textofext', '_aiida_hash': AnyValue()}
 
         # Now I check for the attributes
         # First I check that nothing has changed
         self.assertEquals({k: v for k, v in a.iterattrs()}, attrs_to_set)
-        self.assertEquals({k: v for k, v in a.iterextras()}, extras_to_set)
+        self.assertEquals({k: v for k, v in a.iterextras()}, all_extras)
 
         # I check then on the 'b' copy
         self.assertEquals({k: v
@@ -808,7 +1001,7 @@ class TestNodeBasic(AiidaTestCase):
         self.assertEquals(self.boolval, a.get_attr('bool'))
         self.assertEquals(a_string, a.get_extra('bool'))
 
-        self.assertEquals(a.get_extras(), {'bool': a_string})
+        self.assertEquals(a.get_extras(), {'bool': a_string, '_aiida_hash': AnyValue()})
 
     def test_get_extras_with_default(self):
         a = Node()
@@ -886,14 +1079,16 @@ class TestNodeBasic(AiidaTestCase):
         for k, v in extras_to_set.iteritems():
             a.set_extra(k, v)
 
+        all_extras = dict(_aiida_hash=AnyValue(), **extras_to_set)
+
         self.assertEquals(set(a.attrs()), set(attrs_to_set.keys()))
-        self.assertEquals(set(a.extras()), set(extras_to_set.keys()))
+        self.assertEquals(set(a.extras()), set(all_extras.keys()))
 
         returned_internal_attrs = {k: v for k, v in a.iterattrs()}
         self.assertEquals(returned_internal_attrs, attrs_to_set)
 
         returned_attrs = {k: v for k, v in a.iterextras()}
-        self.assertEquals(returned_attrs, extras_to_set)
+        self.assertEquals(returned_attrs, all_extras)
 
     def test_versioning(self):
         """
@@ -950,10 +1145,12 @@ class TestNodeBasic(AiidaTestCase):
             'further': 267,
         }
 
+        all_extras = dict(_aiida_hash=AnyValue(), **extras_to_set)
+
         for k, v in extras_to_set.iteritems():
             a.set_extra(k, v)
 
-        self.assertEquals({k: v for k, v in a.iterextras()}, extras_to_set)
+        self.assertEquals({k: v for k, v in a.iterextras()}, all_extras)
 
         # I pregenerate it, it cannot change during iteration
         list_keys = list(extras_to_set.keys())
@@ -961,8 +1158,8 @@ class TestNodeBasic(AiidaTestCase):
             # I delete one by one the keys and check if the operation is
             # performed correctly
             a.del_extra(k)
-            del extras_to_set[k]
-            self.assertEquals({k: v for k, v in a.iterextras()}, extras_to_set)
+            del all_extras[k]
+            self.assertEquals({k: v for k, v in a.iterextras()}, all_extras)
 
     def test_replace_extras_1(self):
         """
@@ -986,6 +1183,7 @@ class TestNodeBasic(AiidaTestCase):
                 'h': 'j'
             }, [9, 8, 7]],
         }
+        all_extras = dict(_aiida_hash=AnyValue(), **extras_to_set)
 
         # I redefine the keys with more complicated data, and
         # changing the data type too
@@ -1004,7 +1202,7 @@ class TestNodeBasic(AiidaTestCase):
         for k, v in extras_to_set.iteritems():
             a.set_extra(k, v)
 
-        self.assertEquals({k: v for k, v in a.iterextras()}, extras_to_set)
+        self.assertEquals({k: v for k, v in a.iterextras()}, all_extras)
 
         for k, v in new_extras.iteritems():
             # I delete one by one the keys and check if the operation is
@@ -1013,8 +1211,8 @@ class TestNodeBasic(AiidaTestCase):
 
         # I update extras_to_set with the new entries, and do the comparison
         # again
-        extras_to_set.update(new_extras)
-        self.assertEquals({k: v for k, v in a.iterextras()}, extras_to_set)
+        all_extras.update(new_extras)
+        self.assertEquals({k: v for k, v in a.iterextras()}, all_extras)
 
     def test_basetype_as_attr(self):
         """
@@ -1147,9 +1345,9 @@ class TestNodeBasic(AiidaTestCase):
 
         self.assertEquals([(i['user__email'], i['content'])
                            for i in comments], [
-                               (self.user_email, 'text'),
-                               (self.user_email, 'text2'),
-                           ])
+                              (self.user_email, 'text'),
+                              (self.user_email, 'text2'),
+                          ])
 
     def test_code_loading_from_string(self):
         """
@@ -1213,22 +1411,22 @@ class TestNodeBasic(AiidaTestCase):
         # Check that you can load it with a simple integer id.
         a2 = Node.get_subclass_from_pk(a1.id)
         self.assertEquals(a1.id, a2.id, "The ids of the stored and loaded node"
-                          "should be equal (since it should be "
-                          "the same node")
+                                        "should be equal (since it should be "
+                                        "the same node")
 
         # Check that you can load it with an id of type long.
         # a3 = Node.get_subclass_from_pk(long(a1.id))
         a3 = Node.get_subclass_from_pk(long(a1.id))
         self.assertEquals(a1.id, a3.id, "The ids of the stored and loaded node"
-                          "should be equal (since it should be "
-                          "the same node")
+                                        "should be equal (since it should be "
+                                        "the same node")
 
         # Check that it manages to load the node even if the id is
         # passed as a string.
         a4 = Node.get_subclass_from_pk(str(a1.id))
         self.assertEquals(a1.id, a4.id, "The ids of the stored and loaded node"
-                          "should be equal (since it should be "
-                          "the same node")
+                                        "should be equal (since it should be "
+                                        "the same node")
 
         # Check that a ValueError exception is raised when a string that can
         # not be casted to integer is passed.
@@ -1410,8 +1608,12 @@ class TestNodeBasic(AiidaTestCase):
             with self.assertRaises(NotExistent):
                 load_node(spec, parent_class=ArrayData)
 
-    def test_load_plugin_safe(self):
-        from aiida.orm import (JobCalculation, CalculationFactory, DataFactory)
+    def test_load_unknown_calculation_type(self):
+        """
+        Test that the loader will choose a common calculation ancestor for an unknown data type.
+        For the case where, e.g., the user doesn't have the necessary plugin.
+        """
+        from aiida.orm import (JobCalculation, CalculationFactory)
 
         ###### for calculation
         calc_params = {
@@ -1421,37 +1623,47 @@ class TestNodeBasic(AiidaTestCase):
 
         TemplateReplacerCalc = CalculationFactory('simpleplugins.templatereplacer')
         testcalc = TemplateReplacerCalc(**calc_params).store()
-        jobcalc = JobCalculation(**calc_params).store()
 
         # compare if plugin exist
-        obj = testcalc.dbnode.get_aiida_class()
+        obj = load_node(uuid=testcalc.uuid)
         self.assertEqual(type(testcalc), type(obj))
 
-        # change node type and save in database again
-        testcalc.dbnode.type = "calculation.job.simpleplugins_tmp.templatereplacer.TemplatereplacerCalculation."
-        testcalc.dbnode.save()
+        # Create a custom calculation type that inherits from JobCalculation
+        class TestCalculation(JobCalculation):
+            pass
+
+        jobcalc = JobCalculation(**calc_params).store()
+        testcalc = TestCalculation(**calc_params).store()
 
         # changed node should return job calc as its plugin is not exist
-        obj = testcalc.dbnode.get_aiida_class()
+        obj = load_node(uuid=testcalc.uuid)
         self.assertEqual(type(jobcalc), type(obj))
 
-        ####### for data
+    def test_load_unknown_data_type(self):
+        """
+        Test that the loader will choose a common data ancestor for an unknown data type.
+        For the case where, e.g., the user doesn't have the necessary plugin.
+        """
+        from aiida.orm import DataFactory
+
         KpointsData = DataFactory('array.kpoints')
         kpoint = KpointsData().store()
         Data = DataFactory("Data")
         data = Data().store()
 
         # compare if plugin exist
-        obj = kpoint.dbnode.get_aiida_class()
+        obj = load_node(uuid=kpoint.uuid)
         self.assertEqual(type(kpoint), type(obj))
 
+        class TestKpointsData(KpointsData):
+            pass
+
         # change node type and save in database again
-        kpoint.dbnode.type = "data.array.kpoints_tmp.KpointsData."
-        kpoint.dbnode.save()
+        test_kpoint = TestKpointsData().store()
 
         # changed node should return data node as its plugin is not exist
-        obj = kpoint.dbnode.get_aiida_class()
-        self.assertEqual(type(data), type(obj))
+        obj = load_node(uuid=test_kpoint.uuid)
+        self.assertEqual(type(kpoint), type(obj))
 
         ###### for node
         n1 = Node().store()
@@ -1459,7 +1671,6 @@ class TestNodeBasic(AiidaTestCase):
         self.assertEqual(type(n1), type(obj))
 
 
-
 class TestSubNodesAndLinks(AiidaTestCase):
 
     def test_cachelink(self):
@@ -1503,7 +1714,7 @@ class TestSubNodesAndLinks(AiidaTestCase):
         self.assertEqual(
             set([(i[0], i[1].uuid)
                  for i in endnode.get_inputs(only_in_db=True, also_labels=True)
-                ]), set())
+                 ]), set())
         self.assertEqual(
             set([(i[0], i[1].uuid)
                  for i in endnode.get_inputs(also_labels=True)]),
@@ -1516,7 +1727,7 @@ class TestSubNodesAndLinks(AiidaTestCase):
         self.assertEqual(
             set([(i[0], i[1].uuid)
                  for i in endnode.get_inputs(only_in_db=True, also_labels=True)
-                ]),
+                 ]),
             set([("N1", n1.uuid), ("N2", n2.uuid), ("N3", n3.uuid), ("N4",
                                                                      n4.uuid)]))
         self.assertEqual(
@@ -1551,7 +1762,7 @@ class TestSubNodesAndLinks(AiidaTestCase):
         self.assertEqual(
             set([(i[0], i[1].uuid)
                  for i in endnode.get_inputs(only_in_db=True, also_labels=True)
-                ]), set([("N1", n1.uuid), ("N2", n2.uuid)]))
+                 ]), set([("N1", n1.uuid), ("N2", n2.uuid)]))
         self.assertEqual(
             set([(i[0], i[1].uuid)
                  for i in endnode.get_inputs(also_labels=True)]),
@@ -1600,13 +1811,13 @@ class TestSubNodesAndLinks(AiidaTestCase):
         n2.add_link_from(n1, "N1", link_type=LinkType.CREATE)
 
         self.assertTrue(n1.has_children, "It should be true since n2 is the "
-                        "child of n1.")
+                                         "child of n1.")
         self.assertFalse(n1.has_parents, "It should be false since n1 doesn't "
-                         "have any parents.")
+                                         "have any parents.")
         self.assertFalse(n2.has_children, "It should be false since n2 "
-                         "doesn't have any children.")
+                                          "doesn't have any children.")
         self.assertTrue(n2.has_parents, "It should be true since n1 is the "
-                        "parent of n2.")
+                                        "parent of n2.")
 
     def test_use_code(self):
         from aiida.orm import JobCalculation
@@ -1649,7 +1860,7 @@ class TestSubNodesAndLinks(AiidaTestCase):
         self.assertEqual(calc.get_code().uuid, code.uuid)
         self.assertEqual(unstoredcalc.get_code().uuid, code.uuid)
 
-    #pylint: disable=unused-variable,no-member,no-self-use
+    # pylint: disable=unused-variable,no-member,no-self-use
     def test_calculation_load(self):
         from aiida.orm import JobCalculation
 
@@ -1969,3 +2180,234 @@ class TestSubNodesAndLinks(AiidaTestCase):
         # more than one input to the same data object!
         with self.assertRaises(ValueError):
             d1.add_link_from(calc2, link_type=LinkType.CREATE)
+
+    def test_node_get_inputs_outputs_link_type_stored(self):
+        """
+        Test that the link_type parameter in get_inputs and get_outputs only
+        returns those nodes with the correct link type for stored nodes
+        """
+        node_origin = Node().store()
+        node_caller = Node().store()
+        node_called = Node().store()
+        node_input = Node().store()
+        node_output = Node().store()
+        node_return = Node().store()
+
+        # Input links of node_origin
+        node_origin.add_link_from(node_caller, label='caller', link_type=LinkType.CALL)
+        node_origin.add_link_from(node_input, label='input', link_type=LinkType.INPUT)
+
+        # Output links of node_origin
+        node_called.add_link_from(node_origin, label='called', link_type=LinkType.CALL)
+        node_output.add_link_from(node_origin, label='output', link_type=LinkType.CREATE)
+        node_return.add_link_from(node_origin, label='return', link_type=LinkType.RETURN)
+
+        # All inputs and outputs
+        self.assertEquals(len(node_origin.get_inputs()), 2)
+        self.assertEquals(len(node_origin.get_outputs()), 3)
+
+        # Link specific inputs
+        self.assertEquals(len(node_origin.get_inputs(link_type=LinkType.CALL)), 1)
+        self.assertEquals(len(node_origin.get_inputs(link_type=LinkType.INPUT)), 1)
+
+        # Link specific outputs
+        self.assertEquals(len(node_origin.get_outputs(link_type=LinkType.CALL)), 1)
+        self.assertEquals(len(node_origin.get_outputs(link_type=LinkType.CREATE)), 1)
+        self.assertEquals(len(node_origin.get_outputs(link_type=LinkType.RETURN)), 1)
+
+    def test_node_get_inputs_link_type_unstored(self):
+        """
+        Test that the link_type parameter in get_inputs only returns those nodes with
+        the correct link type for unstored nodes. We don't check this analogously for
+        get_outputs because there is not output links cache
+        """
+        node_origin = Node()
+        node_caller = Node()
+        node_input = Node()
+
+        # Input links of node_origin
+        node_origin.add_link_from(node_caller, label='caller', link_type=LinkType.CALL)
+        node_origin.add_link_from(node_input, label='input', link_type=LinkType.INPUT)
+
+        # All inputs and outputs
+        self.assertEquals(len(node_origin.get_inputs()), 2)
+
+        # Link specific inputs
+        self.assertEquals(len(node_origin.get_inputs(link_type=LinkType.CALL)), 1)
+        self.assertEquals(len(node_origin.get_inputs(link_type=LinkType.INPUT)), 1)
+
+
+class AnyValue(object):
+    """
+    Helper class that compares equal to everything.
+    """
+
+    def __eq__(self, other):
+        return True
+
+
+class TestNodeDeletion(AiidaTestCase):
+
+    def _check_existence(self, uuids_check_existence, uuids_check_deleted):
+        """
+        I get 2 lists of uuids
+
+        :param list uuids_check_existence: The list of uuids that have to exist
+        :param list uuids_check_deleted: The list of uuids that should not exist,
+            I check that NotExistent is raised.
+        """
+        from aiida.common.exceptions import NotExistent
+        for uuid in uuids_check_existence:
+            # This will raise if node is not existent:
+            load_node(uuid)
+        for uuid in uuids_check_deleted:
+            # I check that it raises
+            with self.assertRaises(NotExistent):
+                load_node(uuid)
+
+    def _create_calls_n_returns_graph(self):
+        """
+        Creates a complicated graph with a master with 1 inputs,
+        2 slaves of it also using that input and an additional 1,
+        producing output that is either returned or not returned by the master.
+        Master also creates one nodes. This allows to check whether the delete_nodes
+        command works as anticipated.
+        """
+        in1, in2, wf, slave1, outp1, outp2, slave2, outp3, outp4 = [Node().store() for i in range(9)]
+        wf.add_link_from(in1, link_type=LinkType.INPUT)
+        slave1.add_link_from(in1, link_type=LinkType.INPUT)
+        slave1.add_link_from(in2, link_type=LinkType.INPUT)
+        slave2.add_link_from(in2, link_type=LinkType.INPUT)
+        slave1.add_link_from(wf, link_type=LinkType.CALL)
+        slave2.add_link_from(wf, link_type=LinkType.CALL)
+        outp1.add_link_from(slave1, link_type=LinkType.CREATE)
+        outp2.add_link_from(slave2, link_type=LinkType.CREATE)
+        outp2.add_link_from(wf, link_type=LinkType.RETURN)
+        outp3.add_link_from(wf, link_type=LinkType.CREATE)
+        outp4.add_link_from(wf, link_type=LinkType.RETURN)
+        return in1, in2, wf, slave1, outp1, outp2, slave2, outp3, outp4
+
+    def test_deletion_simple(self):
+        """
+        I'm setting up a sequence of nodes connected by data provenance links.
+        Testing whether I will delete the right ones.
+        """
+        nodes = [Node().store() for i in range(15)]
+        uuids_check_existence = [n.uuid for n in nodes[:3]]
+        uuids_check_deleted = [n.uuid for n in nodes[3:]]
+
+        # Now I am linking the nodes in a branched network
+        # Connecting nodes 1,2,3 to 0
+        for i in range(1, 4):
+            nodes[i].add_link_from(nodes[0], link_type=LinkType.INPUT)
+        # Connecting nodes 4,5,6 to 3
+        for i in range(4, 7):
+            nodes[i].add_link_from(nodes[3], link_type=LinkType.CREATE)
+        # Connecting nodes 7,8 to 4
+        for i in range(7, 9):
+            nodes[i].add_link_from(nodes[4], link_type=LinkType.INPUT)
+        # Connecting nodes 9 to 5
+        for i in range(9, 10):
+            nodes[i].add_link_from(nodes[5], link_type=LinkType.INPUT)
+        for i in range(10, 14):
+            nodes[i + 1].add_link_from(nodes[i], link_type=LinkType.INPUT)
+
+        with Capturing():
+            delete_nodes((nodes[3].pk, nodes[10].pk), force=True, verbosity=2)
+
+        self._check_existence(uuids_check_existence, uuids_check_deleted)
+
+    def test_deletion_with_calls_with_returns(self):
+        """
+        Checking the case where I follow calls and return links for deletion
+        """
+        in1, in2, wf, slave1, outp1, outp2, slave2, outp3, outp4 = self._create_calls_n_returns_graph()
+        # The inputs are not harmed.
+        uuids_check_existence = (in1.uuid, in2.uuid)
+        # the slaves and their outputs have to disappear since calls are followed!
+        uuids_check_deleted = [n.uuid for n in (wf, slave1, outp1, outp2, outp3, slave2, outp4)]
+
+        with Capturing():
+            delete_nodes([wf.pk], verbosity=2, force=True, follow_calls=True, follow_returns=True)
+
+        self._check_existence(uuids_check_existence, uuids_check_deleted)
+
+    def test_deletion_with_calls_no_returns(self):
+        """
+        Checking the case where I follow calls and not return links for deletion
+        """
+        in1, in2, wf, slave1, outp1, outp2, slave2, outp3, outp4 = self._create_calls_n_returns_graph()
+        # The inputs are not harmed.
+        uuids_check_existence = (in1.uuid, in2.uuid, outp4.uuid)
+        # the slaves and their outputs have to disappear since calls are followed!
+        uuids_check_deleted = [n.uuid for n in (wf, slave1, outp1, outp2, outp3, slave2)]
+
+        with Capturing():
+            delete_nodes([wf.pk], verbosity=2, force=True, follow_calls=True, follow_returns=False)
+
+        self._check_existence(uuids_check_existence, uuids_check_deleted)
+
+    def test_deletion_no_calls_no_returns(self):
+        """
+        Checking the case where I don't follow calls and also not return links for deletion
+        """
+        in1, in2, wf, slave1, outp1, outp2, slave2, outp3, outp4 = self._create_calls_n_returns_graph()
+        # I don't follow calls, so the slaves and their output are unharmed, as well as input
+        uuids_check_existence = [n.uuid for n in (in1, in2, slave1, outp1, outp2, slave2, outp4)]
+        # the wf and it's direct output
+        uuids_check_deleted = [n.uuid for n in (wf, outp3)]
+
+        with Capturing():
+            delete_nodes([wf.pk], verbosity=2, force=True, follow_calls=False, follow_returns=False)
+
+        self._check_existence(uuids_check_existence, uuids_check_deleted)
+        self._check_existence(uuids_check_existence, uuids_check_deleted)
+
+    def test_deletion_no_calls_with_returns(self):
+        """
+        Checking the case where I follow returns and not calls for deletion
+        """
+        in1, in2, wf, slave1, outp1, outp2, slave2, outp3, outp4 = self._create_calls_n_returns_graph()
+        # I don't follow calls, so the slaves and their output are unharmed, as well as input
+        uuids_check_existence = [n.uuid for n in (in1, in2, slave1, outp1, slave2)]
+        # the wf and it's direct output and what it returned
+        uuids_check_deleted = [n.uuid for n in (wf, outp3, outp2, outp4)]
+
+        with Capturing():
+            delete_nodes([wf.pk], verbosity=2, force=True, follow_calls=False, follow_returns=True)
+
+        self._check_existence(uuids_check_existence, uuids_check_deleted)
+        self._check_existence(uuids_check_existence, uuids_check_deleted)
+
+    def test_deletion_with_returns_n_loops(self):
+        """
+        Setting up a simple loop, to check that the following doesn't go bananas.
+        """
+        in1, in2, wf = [Node().store() for i in range(3)]
+        wf.add_link_from(in1, link_type=LinkType.INPUT)
+        wf.add_link_from(in2, link_type=LinkType.INPUT)
+        in2.add_link_from(wf, link_type=LinkType.RETURN)
+
+        uuids_check_existence = (in1.uuid,)
+        uuids_check_deleted = [n.uuid for n in (wf, in2)]
+
+        with Capturing():
+            delete_nodes([wf.pk], verbosity=2, force=True, follow_returns=True)
+
+        self._check_existence(uuids_check_existence, uuids_check_deleted)
+
+    def test_delete_called_but_not_caller(self):
+        """
+        Check that deleting a Calculation that was called by another Calculation which won't be
+        deleted works, even though it will raise a warning
+        """
+        caller, called = [Calculation().store() for i in range(2)]
+        called.add_link_from(caller, link_type=LinkType.CALL)
+
+        uuids_check_existence = (caller.uuid, )
+        uuids_check_deleted = [n.uuid for n in (called, )]
+
+        with Capturing():
+            delete_nodes([called.pk], verbosity=2, force=True, follow_returns=True)
+
+        self._check_existence(uuids_check_existence, uuids_check_deleted)
