@@ -464,24 +464,24 @@ class TestTcodDbExporter(AiidaTestCase):
             '_cell_length_b',
             '_cell_length_c',
             '_chemical_formula_sum',
-            '_symmetry_Int_Tables_number',
             '_symmetry_equiv_pos_as_xyz',
-            '_symmetry_space_group_name_H-M',
-            '_symmetry_space_group_name_Hall'
+            '_symmetry_int_tables_number',
+            '_symmetry_space_group_name_h-m',
+            '_symmetry_space_group_name_hall'
         ]
 
         tcod_file_tags = [
             '_tcod_content_encoding_id',
             '_tcod_content_encoding_layer_id',
             '_tcod_content_encoding_layer_type',
-            '_tcod_file_URI',
             '_tcod_file_content_encoding',
             '_tcod_file_contents',
             '_tcod_file_id',
             '_tcod_file_md5sum',
             '_tcod_file_name',
             '_tcod_file_role',
-            '_tcod_file_sha1sum'
+            '_tcod_file_sha1sum',
+            '_tcod_file_uri',
         ]
 
         # Not stored and not to be stored:
@@ -500,7 +500,7 @@ class TestTcodDbExporter(AiidaTestCase):
         v = export_values(td, trajectory_index=1, store=True)
         self.assertEqual(sorted(v['0'].keys()),
                          expected_tags + tcod_file_tags)
-
+        
         # Both stored and expected to be stored:
         td = TrajectoryData(structurelist=structurelist)
         td.store()
