@@ -7,12 +7,26 @@
 # For further information on the license, see the LICENSE.txt file        #
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
-from passlib.context import CryptContext
-import random
 import hashlib
+import numbers
+import random
 import time
+import uuid
 from datetime import datetime
 
+from passlib.context import CryptContext
+
+try: # Python3
+    from functools import singledispatch
+    from collections import abc
+except ImportError: # Python2
+    from singledispatch import singledispatch
+    import collections as abc
+
+import numpy as np
+
+from .folders import Folder
+
 """
 Here we define a single password hashing instance for the full AiiDA.
 """
@@ -112,14 +126,14 @@ def make_hash_with_type(type_chr, string_to_hash):
     """
     return hashlib.sha224("{}{}".format(type_chr, string_to_hash)).hexdigest()
 
-def make_hash(object_to_hash, float_precision=12):
+@singledispatch
+def make_hash(object_to_hash, **kwargs):
     """
     Makes a hash from a dictionary, list, tuple or set to any level, that contains
     only other hashable or nonhashable types (including lists, tuples, sets, and
     dictionaries).
 
     :param object_to_hash: the object to hash
-    :param int float_precision: the precision when converting floats to strings
 
     :returns: a unique hash
 
@@ -181,66 +195,132 @@ def make_hash(object_to_hash, float_precision=12):
     the string of dictionary do not suffice if we want to check for equality
     of dictionaries using hashes.
     """
-    import numpy as np
+    raise ValueError("Value of type {} cannot be hashed".format(
+        type(object_to_hash))
+    )
 
-    if isinstance(object_to_hash, (tuple, list)):
-        hashes = tuple([
-                make_hash(_, float_precision=float_precision)
-                for _
-                in object_to_hash
-            ])
-        # We treat lists and tuples as if they are the same thing,
-        # but I think this is OK
-        return make_hash_with_type('L', "".join(hashes))
+@make_hash.register(abc.Sequence)
+def _(sequence, **kwargs):
+    hashes = tuple([
+        make_hash(x, **kwargs) for x in sequence
+    ])
+    return make_hash_with_type('L', ",".join(hashes))
 
-    elif isinstance(object_to_hash, set):
-        hashes = tuple([
-                make_hash(_, float_precision=float_precision)
-                for _
-                in sorted(object_to_hash)
-            ])
-        return make_hash_with_type('S', "".join(hashes))
+@make_hash.register(abc.Set)
+def _(object_to_hash, **kwargs):
+    hashes = tuple([
+            make_hash(x, **kwargs)
+            for x
+            in sorted(object_to_hash)
+        ])
+    return make_hash_with_type('S', ",".join(hashes))
 
-    elif isinstance(object_to_hash, dict):
-        hashed_dictionary = {
-            k: make_hash(v, float_precision=float_precision)
-            for k,v
-            in object_to_hash.items()
-        }
-        return make_hash_with_type(
-            'D', make_hash(sorted(
-                    hashed_dictionary.items()), float_precision=float_precision
-                )
+@make_hash.register(abc.Mapping)
+def _(mapping, **kwargs):
+    hashed_dictionary = {
+        k: make_hash(v, **kwargs)
+        for k,v
+        in mapping.items()
+    }
+    return make_hash_with_type(
+        'D',
+        make_hash(sorted(hashed_dictionary.items()), **kwargs)
+    )
+
+@make_hash.register(numbers.Real)
+def _(object_to_hash, **kwargs):
+    return make_hash_with_type(
+            'f',
+            truncate_float64(object_to_hash).tobytes()
+        )
+
+@make_hash.register(numbers.Complex)
+def _(object_to_hash, **kwargs):
+    return make_hash_with_type(
+        'c',
+        ','.join([
+            make_hash(object_to_hash.real, **kwargs),
+            make_hash(object_to_hash.imag, **kwargs)
+        ])
+    )
+
+@make_hash.register(numbers.Integral)
+def _(object_to_hash, **kwargs):
+    return make_hash_with_type('i', str(object_to_hash))
+
+@make_hash.register(basestring)
+def _(object_to_hash, **kwargs):
+    return make_hash_with_type('s', object_to_hash)
+
+@make_hash.register(bool)
+def _(object_to_hash, **kwargs):
+    return make_hash_with_type('b', str(object_to_hash))
+
+@make_hash.register(type(None))
+def _(object_to_hash, **kwargs):
+    return make_hash_with_type('n', str(object_to_hash))
+
+@make_hash.register(datetime)
+def _(object_to_hash, **kwargs):
+    return make_hash_with_type('d', str(object_to_hash))
+
+@make_hash.register(uuid.UUID)
+def _(object_to_hash, **kwargs):
+    return make_hash_with_type('u', str(object_to_hash))
+
+
+@make_hash.register(Folder)
+def _(folder, **kwargs):
+    # make sure file is closed after being read
+    def _read_file(folder, name):
+        with folder.open(name) as f:
+            return f.read()
+
+    ignored_folder_content = kwargs.get('ignored_folder_content', [])
+
+    return make_hash_with_type(
+        'pd',
+        make_hash([
+            (
+                name,
+                folder.get_subfolder(name) if folder.isdir(name) else
+                make_hash_with_type('pf', _read_file(folder, name))
             )
+            for name in sorted(folder.get_content_list())
+            if name not in ignored_folder_content
+        ], **kwargs)
+    )
 
-    elif isinstance(object_to_hash, float):
+@make_hash.register(np.ndarray)
+def _(object_to_hash, **kwargs):
+    if object_to_hash.dtype == np.float64:
         return make_hash_with_type(
-                'f','{:.{precision}f}'.format(
-                        object_to_hash, precision=float_precision
-                )
-            )
-    # If is numpy:
-    elif type(object_to_hash).__module__ == np.__name__:
-        return make_hash_with_type('N', str(object_to_hash))
-
-    elif isinstance(object_to_hash, basestring):
-        return make_hash_with_type('s', object_to_hash)
-
-    elif isinstance(object_to_hash, bool): # bool must come before int
-        # I prefer to be sure of what I hash instead of using 'str'
-        return make_hash_with_type('b', "True" if object_to_hash else "False")
-
-    elif object_to_hash is None:
-        return make_hash_with_type('n', "None")
-
-    elif isinstance(object_to_hash, int):
-        return make_hash_with_type('i', str(object_to_hash))
-    elif isinstance(object_to_hash, long):
-        return make_hash_with_type('l', str(object_to_hash))
-
-    elif isinstance(object_to_hash, datetime):
-        return make_hash_with_type('d', str(object_to_hash))
-    # Possibly add more types here, as needed
+            'af',
+            make_hash(truncate_array64(object_to_hash).tobytes(), **kwargs)
+        )
+    elif object_to_hash.dtype == np.complex128:
+        return make_hash_with_type(
+            'ac',
+            make_hash([
+                object_to_hash.real,
+                object_to_hash.imag
+            ], **kwargs)
+        )
     else:
-        raise ValueError("Value of type {} cannot be hashed".format(
-                type(object_to_hash)))
+        return make_hash_with_type(
+            'ao',
+            make_hash(object_to_hash.tobytes(), **kwargs)
+        )
+
+def truncate_float64(x, num_bits=4):
+    mask = ~(2**num_bits - 1)
+    int_repr = np.float64(x).view(np.int64)
+    masked_int = int_repr & mask
+    truncated_x = masked_int.view(np.float64)
+    return truncated_x
+
+def truncate_array64(x, num_bits=4):
+    mask = ~(2**num_bits - 1)
+    int_array = np.array(x, dtype=np.float64).view(np.int64)
+    masked_array = int_array & mask
+    return masked_array.view(np.float64)
