@@ -476,6 +476,7 @@ class TrajectoryData(ArrayData):
         import CifFile
         from aiida.orm.data.cif \
             import ase_loops, cif_from_ase, pycifrw_from_cif
+        from aiida.common.utils import HiddenPrints
 
         cif = ""
         indices = range(self.numsteps)
@@ -485,7 +486,8 @@ class TrajectoryData(ArrayData):
             structure = self.get_step_structure(idx)
             ciffile = pycifrw_from_cif(cif_from_ase(structure.get_ase()),
                                        ase_loops)
-            cif = cif + ciffile.WriteOut()
+            with HiddenPrints():
+                cif = cif + ciffile.WriteOut()
         return cif.encode('utf-8'), {}
 
     def _prepare_tcod(self, main_file_name="", **kwargs):
@@ -532,7 +534,7 @@ class TrajectoryData(ArrayData):
             decided not to implement it at all but require it to be set
             explicitly.
 
-        .. usage::
+        Usage::
 
             from aiida.orm.data.array.trajectory import TrajectoryData
 
@@ -572,7 +574,7 @@ class TrajectoryData(ArrayData):
 
         .. note:: The steps and symbols must be set manually before calling this
             import function as a consistency measure. See also comment for
-            :py:meth:`._import_xy_pos`
+            :py:meth:`._parse_xyz_pos`
         """
 
         from aiida.common.exceptions import ValidationError
