@@ -11,6 +11,7 @@
 # pylint: disable=invalid-name,too-many-locals,too-many-statements
 from aiida.orm.data.singlefile import SinglefileData
 from aiida.orm.calculation.inline import optional_inline
+from aiida.common.utils import HiddenPrints
 
 ase_loops = {
     '_atom_site': [
@@ -52,6 +53,7 @@ def has_pycifrw():
     # pylint: disable=unused-variable
     try:
         import CifFile
+        from CifFile import CifBlock
     except ImportError:
         return False
     return True
@@ -206,6 +208,7 @@ def cif_from_ase(ase, full_occupancies=False, add_fake_biso=False):
     return datablocks
 
 
+# pylint: disable=too-many-branches
 def pycifrw_from_cif(datablocks, loops=None, names=None):
     """
     Constructs PyCifRW's CifFile from an array of CIF datablocks.
@@ -216,11 +219,18 @@ def pycifrw_from_cif(datablocks, loops=None, names=None):
     :return: CifFile
     """
     import CifFile
+    from CifFile import CifBlock
 
     if loops is None:
         loops = dict()
 
     cif = CifFile.CifFile()
+    try:
+        cif.set_grammar("1.1")
+    except AttributeError:
+        # if no grammar can be set, we assume it's 1.1 (widespread standard)
+        pass
+
     if names and len(names) < len(datablocks):
         raise ValueError("Not enough names supplied for "
                          "datablocks: {} (names) < "
@@ -229,8 +239,8 @@ def pycifrw_from_cif(datablocks, loops=None, names=None):
         name = str(i)
         if names:
             name = names[i]
-        cif.NewBlock(name)
-        datablock = cif[name]
+        datablock = CifBlock()
+        cif[name] = datablock
         for loopname in loops.keys():
             loopdata = ([[]], [[]])
             row_size = None
@@ -252,6 +262,10 @@ def pycifrw_from_cif(datablocks, loops=None, names=None):
                 datablock.AddCifItem(loopdata)
         for tag in sorted(values.keys()):
             datablock[tag] = values[tag]
+            # create automatically a loop for non-scalar values
+            if isinstance(values[tag],
+                          (tuple, list)) and tag not in loops.keys():
+                datablock.CreateLoop([tag])
     return cif
 
 
@@ -286,7 +300,8 @@ def refine_inline(node):
     refined_atoms, symmetry = ase_refine_cell(original_atoms)
 
     cif = CifData(ase=refined_atoms)
-    cif.values.dictionary[name] = cif.values.dictionary.pop(str(0))
+    if name != str(0):
+        cif.values.rename(str(0), name)
 
     # Remove all existing symmetry tags before overwriting:
     for tag in symmetry_tags:
@@ -349,9 +364,9 @@ class CifData(SinglefileData):
         when setting ``ase`` or ``values``, a physical CIF file is generated
         first, the values are updated from the physical CIF file.
     """
-    _set_incompatibilities = [("ase", "file"), ("ase", "values"), ("file",
-                                                                   "values")]
-    _scan_types = ['default', 'flex']
+    _set_incompatibilities = [('ase', 'file'), ('ase', 'values'), ('file',
+                                                                   'values')]
+    _scan_types = ['standard', 'flex']
     _parse_policies = ['eager', 'lazy']
 
     @property
@@ -362,8 +377,8 @@ class CifData(SinglefileData):
         """
         parent_dict = super(CifData, self)._set_defaults
         parent_dict.update({
-            "scan_type": self._scan_types[0],
-            "parse_policy": 'eager',
+            'scan_type': self._scan_types[0],
+            'parse_policy': 'eager',
         })
 
         return parent_dict
@@ -469,10 +484,16 @@ class CifData(SinglefileData):
             self._get_folder_pathsubfolder.open(self.filename), **kwargs)
 
     def set_ase(self, aseatoms):
+        """
+        Set the contents of the CifData starting from an ASE atoms object
+
+        :param aseatoms: the ASE atoms object
+        """
         import tempfile
         cif = cif_from_ase(aseatoms)
         with tempfile.NamedTemporaryFile() as f:
-            f.write(pycifrw_from_cif(cif, loops=ase_loops).WriteOut())
+            with HiddenPrints():
+                f.write(pycifrw_from_cif(cif, loops=ase_loops).WriteOut())
             f.flush()
             self.set_file(f.name)
 
@@ -490,11 +511,16 @@ class CifData(SinglefileData):
         if self._values is None:
             try:
                 import CifFile
+                from CifFile import CifBlock
             except ImportError as e:
                 raise ImportError(
                     str(e) + '. You need to install the PyCifRW package.')
-            self._values = CifFile.ReadCif(
+
+            c = CifFile.ReadCif(
                 self.get_file_abs_path(), scantype=self.get_attr('scan_type'))
+            for k, v in c.items():
+                c.dictionary[k] = CifBlock(v)
+            self._values = c
         return self._values
 
     def set_values(self, values):
@@ -509,7 +535,8 @@ class CifData(SinglefileData):
         """
         import tempfile
         with tempfile.NamedTemporaryFile() as f:
-            f.write(values.WriteOut())
+            with HiddenPrints():
+                f.write(values.WriteOut())
             f.flush()
             self.set_file(f.name)
 
@@ -582,7 +609,7 @@ class CifData(SinglefileData):
         does not yet support the CIF2 format as of 02/2018.
         See the CifFile.ReadCif function
 
-        :param scan_type: Either 'default' or 'flex' (see _scan_types)
+        :param scan_type: Either 'standard' or 'flex' (see _scan_types)
         """
         if scan_type in self._scan_types:
             self._set_attr('scan_type', scan_type)
