@@ -182,7 +182,21 @@ def _create_weights_tuple(weights):
         weights_tuple = tuple(float(i) for i in weights)
     return weights_tuple
 
+    
+def create_automatic_kind_name(symbols,weights):
+    """
+    Create a string obtained with the symbols appended one
+    after the other, without spaces, in alphabetical order;
+    if the site has a vacancy, a X is appended at the end too.
+    """
+    sorted_symbol_list = list(set(symbols))
+    sorted_symbol_list.sort()  # In-place sort
+    name_string = "".join(sorted_symbol_list)
+    if has_vacancies(weights):
+        name_string += "X"
+    return name_string
 
+            
 def validate_weights_tuple(weights_tuple, threshold):
     """
     Validates the weight of the atomic kinds.
@@ -529,24 +543,9 @@ def get_formula(symbol_list, mode='hill', separator=""):
                          'reduce, count or count_compact')
 
     if mode in ['hill_compact', 'count_compact']:
-
-        def gcd_list(int_list):
-            """
-            Recursive function to get the greatest common divisor of
-            a list of integers
-            """
-            from fractions import gcd
-            if len(int_list) == 1:
-                return int_list[0]
-            elif len(int_list) == 2:
-                return gcd(int_list[0], int_list[1])
-            else:
-                the_int_list = int_list[2:]
-                the_int_list.append(gcd(int_list[0], int_list[1]))
-                return gcd_list(the_int_list)
-
-        the_gcd = gcd_list([e[0] for e in the_symbol_list])
-        the_symbol_list = [[e[0] / the_gcd, e[1]] for e in the_symbol_list]
+        from fractions import gcd
+        the_gcd = reduce(gcd,[e[0] for e in the_symbol_list])
+        the_symbol_list = [[e[0]/the_gcd,e[1]] for e in the_symbol_list]
 
     return get_formula_from_symbol_list(the_symbol_list, separator=separator)
 
@@ -706,6 +705,47 @@ def _get_cif_ase_inline(struct=None, parameters=None):
     return {'cif': cif}
 
 
+def atom_kinds_to_html(atom_kind):
+    """
+
+    Construct in html format
+
+    an alloy with 0.5 Ge, 0.4 Si and 0.1 vacancy is represented as
+    Ge<sub>0.5</sub> + Si<sub>0.4</sub> + vacancy<sub>0.1</sub>
+
+    Args:
+        atom_kind: a string with the name of the atomic kind, as printed by
+        kind.get_symbols_string(), e.g. Ba0.80Ca0.10X0.10
+
+    Returns:
+        html code for rendered formula
+    """
+
+    # Parse the formula (TODO can be made more robust though never fails if
+    # it takes strings generated with kind.get_symbols_string())
+    import re
+    elements = re.findall(r'([A-Z][a-z]*)([0-1][.[0-9]*]?)?', atom_kind)
+
+    # Compose the html string
+    html_formula_pieces = []
+
+    for element in elements:
+
+        # replace element X by 'vacancy'
+        species = element[0] if element[0] != 'X' else 'vacancy'
+        weight = element[1] if element[1] != '' else None
+
+        if weight is not None:
+            html_formula_pieces.append(species + '<sub>' + weight +
+                                   '</sub>')
+        else:
+            html_formula_pieces.append(species)
+
+    html_formula = ' + '.join(html_formula_pieces)
+
+    return html_formula
+
+
 class StructureData(Data):
     """
     This class contains the information about a given structure, i.e. a
@@ -830,17 +870,56 @@ class StructureData(Data):
 
         .. note:: periodic boundary conditions are set to True in all
             three directions.
-        .. note:: Requires the pymatgen module (version >= 3.0.13, usage
+        .. note:: Requires the pymatgen module (version >= 3.3.5, usage
             of earlier versions may cause errors).
+        
+        :raise ValueError: if there are partial occupancies together with spins.
         """
+        def build_kind_name(species_and_occu):
+            """
+            Build a kind name from a pymatgen Composition, including an
+            additional ordinal if spin is included, e.g. it returns '<element>1'
+            for an atom with spin<0 and '<element>2' for an element with spin>0,
+            ortherwise (no spin) it returns simply '<element>'
+            :param specie: a pymatgen specie
+            :return: a string
+            """
+            has_spin = any([specie.as_dict().get('properties', {}).get('spin', 0) != 0
+                            for specie in species_and_occu.keys()])
+
+            if has_spin and (len(species_and_occu.items()) > 1
+                             or any([weight != 1.0 for weight in species_and_occu.values()])):
+                raise ValueError('Cannot set partial occupancies and spins at the same time')
+
+            if not has_spin:
+                return None
+
+            spin = species_and_occu.keys()[0].as_dict().get('properties', {}).get('spin', 0)
+
+            if spin < 0:
+                return specie.symbol + '1'
+            else:
+                return specie.symbol + '2'
+        
         self.cell = struct.lattice.matrix.tolist()
         self.pbc = [True, True, True]
         self.clear_kinds()
         for site in struct.sites:
-            self.append_atom(
-                symbols=[x[0].symbol for x in site.species_and_occu.items()],
-                weights=[x[1] for x in site.species_and_occu.items()],
-                position=site.coords.tolist())
+            if 'kind_name' in site.properties:
+                kind_name = site.properties['kind_name']
+            else:
+                kind_name = build_kind_name(site.species_and_occu)
+
+            inputs = {
+                'symbols': [x[0].symbol for x in site.species_and_occu.items()],
+                'weights': [x[1] for x in site.species_and_occu.items()],
+                'position': site.coords.tolist()
+            }
+
+            if kind_name is not None:
+                inputs['name'] = kind_name
+
+            self.append_atom(**inputs)
 
     def _validate(self):
         """
@@ -938,6 +1017,79 @@ class StructureData(Data):
         from aiida.tools.dbexporters.tcod import export_cif
         return export_cif(self, **kwargs).encode('utf-8'), {}
 
+    def _prepare_chemdoodle(self, main_file_name=""):
+        """
+        Write the given structure to a string of format required by ChemDoodle.
+        """
+        import numpy as np
+        from itertools import product
+        import json
+
+        supercell_factors=[1, 1, 1]
+
+        # Get cell vectors and atomic position
+        lattice_vectors = np.array(self.get_attr('cell'))
+        base_sites = self.get_attr('sites')
+
+        start1 = -int(supercell_factors[0] / 2)
+        start2 = -int(supercell_factors[1] / 2)
+        start3 = -int(supercell_factors[2] / 2)
+
+        stop1 = start1 + supercell_factors[0]
+        stop2 = start2 + supercell_factors[1]
+        stop3 = start3 + supercell_factors[2]
+
+        grid1 = range(start1, stop1)
+        grid2 = range(start2, stop2)
+        grid3 = range(start3, stop3)
+
+        atoms_json = []
+
+        # Manual recenter of the structure
+        center = (lattice_vectors[0] + lattice_vectors[1] +
+                  lattice_vectors[2]) / 2.
+
+        for ix, iy, iz in product(grid1, grid2, grid3):
+            for base_site in base_sites:
+                shift = (ix * lattice_vectors[0] + iy * lattice_vectors[1] + \
+                         iz * lattice_vectors[2] - center).tolist()
+
+                kind_name = base_site['kind_name']
+                kind_string = self.get_kind(kind_name).get_symbols_string()
+
+                atoms_json.append(
+                    {'l': kind_string,
+                     'x': base_site['position'][0] + shift[0],
+                     'y': base_site['position'][1] + shift[1],
+                     'z': base_site['position'][2] + shift[2],
+                     # 'atomic_elements_html': kind_string
+                     'atomic_elements_html': atom_kinds_to_html(kind_string)
+                     })
+
+        cell_json = {
+            "t": "UnitCell",
+            "i": "s0",
+            "o": (-center).tolist(),
+            "x": (lattice_vectors[0] - center).tolist(),
+            "y": (lattice_vectors[1] - center).tolist(),
+            "z": (lattice_vectors[2] - center).tolist(),
+            "xy": (lattice_vectors[0] + lattice_vectors[1]
+                   - center).tolist(),
+            "xz": (lattice_vectors[0] + lattice_vectors[2]
+                   - center).tolist(),
+            "yz": (lattice_vectors[1] + lattice_vectors[2]
+                   - center).tolist(),
+            "xyz": (lattice_vectors[0] + lattice_vectors[1]
+                    + lattice_vectors[2] - center).tolist(),
+        }
+
+        return_dict = {"s": [cell_json],
+                        "m": [{"a": atoms_json}],
+                        "units": '&Aring;'
+                        }
+
+        return json.dumps(return_dict), {}
+
     def _prepare_xyz(self, main_file_name=""):
         """
         Write the given structure to a string of format XYZ.
@@ -952,8 +1104,8 @@ class StructureData(Data):
         return_list = ["{}".format(len(sites))]
         return_list.append('Lattice="{} {} {} {} {} {} {} {} {}" pbc="{} {} {}"'.format(
             cell[0][0], cell[0][1], cell[0][2],
-            cell[0][0], cell[0][1], cell[0][2],
-            cell[0][0], cell[0][1], cell[0][2],
+            cell[1][0], cell[1][1], cell[1][2],
+            cell[2][0], cell[2][1], cell[2][2],
             self.pbc[0], self.pbc[1], self.pbc[2]
         ))
         for site in sites:
@@ -1143,20 +1295,36 @@ class StructureData(Data):
         """
         return self._get_object_ase()
 
-    def get_pymatgen(self):
+    def get_pymatgen(self,**kwargs):
         """
         Get pymatgen object. Returns Structure for structures with
         periodic boundary conditions (in three dimensions) and Molecule
         otherwise.
+        :param add_spin: True to add the spins to the pymatgen structure.
+        Default is False (no spin added).
+
+        .. note:: The spins are set according to the following rule:
+            
+            * if the kind name ends with 1 -> spin=+1
+            
+            * if the kind name ends with 2 -> spin=-1
 
         .. note:: Requires the pymatgen module (version >= 3.0.13, usage
             of earlier versions may cause errors).
         """
-        return self._get_object_pymatgen()
+        return self._get_object_pymatgen(**kwargs)
 
-    def get_pymatgen_structure(self):
+    def get_pymatgen_structure(self,**kwargs):
         """
         Get the pymatgen Structure object.
+        :param add_spin: True to add the spins to the pymatgen structure.
+        Default is False (no spin added).
+
+        .. note:: The spins are set according to the following rule:
+            
+            * if the kind name ends with 1 -> spin=+1
+            
+            * if the kind name ends with 2 -> spin=-1
 
         .. note:: Requires the pymatgen module (version >= 3.0.13, usage
             of earlier versions may cause errors).
@@ -1167,7 +1335,7 @@ class StructureData(Data):
         :raise ValueError: if periodic boundary conditions do not hold
           in at least one dimension of real space.
         """
-        return self._get_object_pymatgen_structure()
+        return self._get_object_pymatgen_structure(**kwargs)
 
     def get_pymatgen_molecule(self):
         """
@@ -1267,7 +1435,7 @@ class StructureData(Data):
               until an unique name is found
 
         .. note :: checks of equality of species are done using
-          the :py:meth:`~Kind.compare_with` method.
+          the :py:meth:`~aiida.orm.data.structure.Kind.compare_with` method.
         """
         aseatom = kwargs.pop('ase', None)
         if aseatom is not None:
@@ -1743,7 +1911,7 @@ class StructureData(Data):
             asecell.append(site.get_ase(kinds=_kinds))
         return asecell
 
-    def _get_object_pymatgen(self):
+    def _get_object_pymatgen(self,**kwargs):
         """
         Converts
         :py:class:`StructureData <aiida.orm.data.structure.StructureData>`
@@ -1756,21 +1924,30 @@ class StructureData(Data):
             of earlier versions may cause errors).
         """
         if self.pbc == (True, True, True):
-            return self._get_object_pymatgen_structure()
+            return self._get_object_pymatgen_structure(**kwargs)
         else:
-            return self._get_object_pymatgen_molecule()
+            return self._get_object_pymatgen_molecule(**kwargs)
 
-    def _get_object_pymatgen_structure(self):
+    def _get_object_pymatgen_structure(self,**kwargs):
         """
         Converts
         :py:class:`StructureData <aiida.orm.data.structure.StructureData>`
         to pymatgen Structure object
+        :param add_spin: True to add the spins to the pymatgen structure.
+        Default is False (no spin added).
+
+        .. note:: The spins are set according to the following rule:
+            
+            * if the kind name ends with 1 -> spin=+1
+            
+            * if the kind name ends with 2 -> spin=-1
 
         :return: a pymatgen Structure object corresponding to this
           :py:class:`StructureData <aiida.orm.data.structure.StructureData>`
           object
         :raise ValueError: if periodic boundary conditions does not hold
-          in at least one dimension of real space
+          in at least one dimension of real space; if there are partial occupancies
+          together with spins (defined by kind names ending with '1' or '2').
 
         .. note:: Requires the pymatgen module (version >= 3.0.13, usage
             of earlier versions may cause errors)
@@ -1782,15 +1959,41 @@ class StructureData(Data):
                              "all three dimensions of real space")
 
         species = []
-        for s in self.sites:
-            k = self.get_kind(s.kind_name)
-            species.append({s: w for s, w in zip(k.symbols, k.weights)})
+        additional_kwargs = {}
+
+        if (kwargs.pop('add_spin',False) and 
+            any([n.endswith('1') or n.endswith('2') for n in self.get_kind_names()])):
+            # case when spins are defined -> no partial occupancy allowed
+            from pymatgen.core.structure import Specie
+            oxidation_state = 0 # now I always set the oxidation_state to zero
+            for s in self.sites:
+                k = self.get_kind(s.kind_name)
+                if len(k.symbols)!=1 or (len(k.weights)!=1 or sum(k.weights)<1.):
+                    raise ValueError("Cannot set partial occupancies and spins "
+                                     "at the same time")
+                species.append(Specie(k.symbols[0],oxidation_state,
+                                      properties={'spin': -1 if k.name.endswith('1')
+                                            else 1 if k.name.endswith('2') else 0}))
+        else:
+            # case when no spin are defined
+            for s in self.sites:
+                k = self.get_kind(s.kind_name)
+                species.append({s: w for s, w in zip(k.symbols, k.weights)})
+            if any([create_automatic_kind_name(self.get_kind(name).symbols,self.get_kind(name).weights)!=name
+                    for name in self.get_site_kindnames()]):
+                    # add "kind_name" as a properties to each site, whenever
+                    # the kind_name cannot be automatically obtained from the symbols
+                additional_kwargs['site_properties'] = {'kind_name': self.get_site_kindnames()}
+        
+        if kwargs:
+            raise ValueError("Unrecognized parameters passed to pymatgen "
+                             "converter: {}".format(kwargs.keys()))
 
         positions = [list(x.position) for x in self.sites]
         return Structure(self.cell, species, positions,
-                         coords_are_cartesian=True)
+                         coords_are_cartesian=True,**additional_kwargs)
 
-    def _get_object_pymatgen_molecule(self):
+    def _get_object_pymatgen_molecule(self,**kwargs):
         """
         Converts
         :py:class:`StructureData <aiida.orm.data.structure.StructureData>`
@@ -1805,6 +2008,10 @@ class StructureData(Data):
         """
         from pymatgen.core.structure import Molecule
 
+        if kwargs:
+            raise ValueError("Unrecognized parameters passed to pymatgen "
+                             "converter: {}".format(kwargs.keys()))
+
         species = []
         for s in self.sites:
             k = self.get_kind(s.kind_name)
@@ -1835,11 +2042,11 @@ class Kind(object):
 
         :param symbols: a single string for the symbol of this site, or a list
                    of symbol strings
-        :param weights (optional): the weights for each atomic species of
+        :param weights: (optional) the weights for each atomic species of
                    this site.
                    If only a single symbol is provided, then this value is
                    optional and the weight is set to 1.
-        :param mass (optional): the mass for this site in atomic mass units.
+        :param mass: (optional) the mass for this site in atomic mass units.
                    If not provided, the mass is set by the
                    self.reset_mass() function.
         :param name: a string that uniquely identifies the kind, and that
@@ -2022,11 +2229,7 @@ class Kind(object):
         after the other, without spaces, in alphabetical order;
         if the site has a vacancy, a X is appended at the end too.
         """
-        sorted_symbol_list = list(set(self.symbols))
-        sorted_symbol_list.sort()  # In-place sort
-        name_string = "".join(sorted_symbol_list)
-        if self.has_vacancies():
-            name_string += "X"
+        name_string = create_automatic_kind_name(self.symbols,self.weights)
         if tag is None:
             self.name = name_string
         else:
