@@ -21,7 +21,8 @@ from aiida.common.exceptions import (InternalError, ModificationNotAllowed,
 from aiida.common.folders import RepositoryFolder
 from aiida.common.links import LinkType
 from aiida.common.utils import get_new_uuid
-from aiida.orm.implementation.general.node import AbstractNode, _NO_DEFAULT
+from aiida.orm.implementation.general.node import AbstractNode, _NO_DEFAULT, _HASH_EXTRA_KEY
+from aiida.orm.implementation.django.computer import Computer
 from aiida.orm.mixins import Sealable
 # from aiida.orm.implementation.django.utils import get_db_columns
 from aiida.orm.implementation.general.utils import get_db_columns
@@ -40,12 +41,6 @@ class Node(AbstractNode):
                 uuid, cls.__name__))
         return node
 
-    @staticmethod
-    def get_db_columns():
-        # from aiida.backends.djsite.db.models import DbNode
-        from aiida.backends.djsite.querybuilder_django.dummy_model import DbNode
-        return get_db_columns(DbNode)
-
     @classmethod
     def get_subclass_from_pk(cls, pk):
         from aiida.backends.djsite.db.models import DbNode
@@ -58,6 +53,29 @@ class Node(AbstractNode):
                 pk, cls.__name__))
         return node
 
+    @classmethod
+    def query(cls, *args, **kwargs):
+        from aiida.backends.djsite.db.models import DbNode
+        if cls._plugin_type_string:
+            if not cls._plugin_type_string.endswith('.'):
+                raise InternalError("The plugin type string does not "
+                                    "finish with a dot??")
+
+            # If it is 'calculation.Calculation.', we want to filter
+            # for things that start with 'calculation.' and so on
+            plug_type = cls._plugin_type_string
+
+            # Remove the implementation.django or sqla part.
+            if plug_type.startswith('implementation.'):
+                plug_type = '.'.join(plug_type.split('.')[2:])
+            pre, sep, _ = plug_type[:-1].rpartition('.')
+            superclass_string = "".join([pre, sep])
+            return DbNode.aiidaobjects.filter(
+                *args, type__startswith=superclass_string, **kwargs)
+        else:
+            # Base Node class, with empty string
+            return DbNode.aiidaobjects.filter(*args, **kwargs)
+
     def __init__(self, **kwargs):
         from aiida.backends.djsite.db.models import DbNode
         super(Node, self).__init__()
@@ -87,7 +105,7 @@ class Node(AbstractNode):
 
             # If this is changed, fix also the importer
             self._repo_folder = RepositoryFolder(section=self._section_name,
-                                                 uuid=self._dbnode.uuid)
+                                                 uuid=self.uuid)
 
         # NO VALIDATION ON __init__ BY DEFAULT, IT IS TOO SLOW SINCE IT OFTEN
         # REQUIRES MULTIPLE DB HITS
@@ -122,38 +140,33 @@ class Node(AbstractNode):
             # stop
             self._set_with_defaults(**kwargs)
 
-    @classmethod
-    def query(cls, *args, **kwargs):
-        from aiida.backends.djsite.db.models import DbNode
-        if cls._plugin_type_string:
-            if not cls._plugin_type_string.endswith('.'):
-                raise InternalError("The plugin type string does not "
-                                    "finish with a dot??")
+    @property
+    def type(self):
+        return self._dbnode.type
 
-            # If it is 'calculation.Calculation.', we want to filter
-            # for things that start with 'calculation.' and so on
-            plug_type = cls._plugin_type_string
+    @property
+    def ctime(self):
+        return self._dbnode.ctime
 
-            # Remove the implementation.django or sqla part.
-            if plug_type.startswith('implementation.'):
-                plug_type = '.'.join(plug_type.split('.')[2:])
-            pre, sep, _ = plug_type[:-1].rpartition('.')
-            superclass_string = "".join([pre, sep])
-            return DbNode.aiidaobjects.filter(
-                *args, type__startswith=superclass_string, **kwargs)
-        else:
-            # Base Node class, with empty string
-            return DbNode.aiidaobjects.filter(*args, **kwargs)
+    @property
+    def mtime(self):
+        return self._dbnode.mtime
+
+    def _get_db_label_field(self):
+        return self._dbnode.label
 
     def _update_db_label_field(self, field_value):
-        self.dbnode.label = field_value
+        self._dbnode.label = field_value
         if not self._to_be_stored:
             with transaction.atomic():
                 self._dbnode.save()
                 self._increment_version_number_db()
 
+    def _get_db_description_field(self):
+        return self._dbnode.description
+
     def _update_db_description_field(self, field_value):
-        self.dbnode.description = field_value
+        self._dbnode.description = field_value
         if not self._to_be_stored:
             with transaction.atomic():
                 self._dbnode.save()
@@ -169,7 +182,7 @@ class Node(AbstractNode):
                 self._add_dblink_from(src, label, link_type)
 
     def _remove_dblink_from(self, label):
-        DbLink.objects.filter(output=self.dbnode, label=label).delete()
+        DbLink.objects.filter(output=self._dbnode, label=label).delete()
 
     def _add_dblink_from(self, src, label=None, link_type=LinkType.UNSPECIFIED):
         from aiida.orm.querybuilder import QueryBuilder
@@ -195,8 +208,8 @@ class Node(AbstractNode):
             # I am linking src->self; a loop would be created if a DbPath exists already
             # in the TC table from self to src
             if QueryBuilder().append(
-                    Node, filters={'id':self.pk}, tag='parent').append(
-                    Node, filters={'id':src.pk}, tag='child', descendant_of='parent').count() > 0:
+                    Node, filters={'id': self.pk}, tag='parent').append(
+                Node, filters={'id': src.pk}, tag='child', descendant_of='parent').count() > 0:
                 raise ValueError(
                     "The link you are attempting to create would generate a loop")
 
@@ -204,7 +217,7 @@ class Node(AbstractNode):
             autolabel_idx = 1
 
             existing_from_autolabels = list(DbLink.objects.filter(
-                output=self.dbnode,
+                output=self._dbnode,
                 label__startswith="link_").values_list('label', flat=True))
             while "link_{}".format(autolabel_idx) in existing_from_autolabels:
                 autolabel_idx += 1
@@ -233,7 +246,7 @@ class Node(AbstractNode):
             # transactions are needed here for Postgresql:
             # https://docs.djangoproject.com/en/1.5/topics/db/transactions/#handling-exceptions-within-postgresql-transactions
             sid = transaction.savepoint()
-            DbLink.objects.create(input=src.dbnode, output=self.dbnode,
+            DbLink.objects.create(input=src._dbnode, output=self._dbnode,
                                   label=label, type=link_type.value)
             transaction.savepoint_commit(sid)
         except IntegrityError as e:
@@ -245,25 +258,35 @@ class Node(AbstractNode):
     def _get_db_input_links(self, link_type):
         from aiida.backends.djsite.db.models import DbLink
 
-        link_filter = {'output': self.dbnode}
+        link_filter = {'output': self._dbnode}
         if link_type is not None:
             link_filter['type'] = link_type.value
         return [(i.label, i.input.get_aiida_class()) for i in
                 DbLink.objects.filter(**link_filter).distinct()]
 
-
     def _get_db_output_links(self, link_type):
         from aiida.backends.djsite.db.models import DbLink
 
-        link_filter = {'input': self.dbnode}
+        link_filter = {'input': self._dbnode}
         if link_type is not None:
             link_filter['type'] = link_type.value
         return ((i.label, i.output.get_aiida_class()) for i in
                 DbLink.objects.filter(**link_filter).distinct())
 
+    def get_computer(self):
+        """
+        Get the computer associated to the node.
+
+        :return: the Computer object or None.
+        """
+        if self._dbnode.dbcomputer is None:
+            return None
+        else:
+            return Computer(dbcomputer=self._dbnode.dbcomputer)
+
     def _set_db_computer(self, computer):
         from aiida.backends.djsite.db.models import DbComputer
-        self.dbnode.dbcomputer = DbComputer.get_dbcomputer(computer)
+        self._dbnode.dbcomputer = DbComputer.get_dbcomputer(computer)
 
     def _set_db_attr(self, key, value):
         """
@@ -277,26 +300,26 @@ class Node(AbstractNode):
         """
         from aiida.backends.djsite.db.models import DbAttribute
 
-        DbAttribute.set_value_for_node(self.dbnode, key, value)
+        DbAttribute.set_value_for_node(self._dbnode, key, value)
         self._increment_version_number_db()
 
     def _del_db_attr(self, key):
         from aiida.backends.djsite.db.models import DbAttribute
-        if not DbAttribute.has_key(self.dbnode, key):
+        if not DbAttribute.has_key(self._dbnode, key):
             raise AttributeError("DbAttribute {} does not exist".format(
                 key))
-        DbAttribute.del_value_for_node(self.dbnode, key)
+        DbAttribute.del_value_for_node(self._dbnode, key)
         self._increment_version_number_db()
 
     def _get_db_attr(self, key):
         from aiida.backends.djsite.db.models import DbAttribute
         return DbAttribute.get_value_for_node(
-            dbnode=self.dbnode, key=key)
+            dbnode=self._dbnode, key=key)
 
     def _set_db_extra(self, key, value, exclusive=False):
         from aiida.backends.djsite.db.models import DbExtra
 
-        DbExtra.set_value_for_node(self.dbnode, key, value,
+        DbExtra.set_value_for_node(self._dbnode, key, value,
                                    stop_if_existing=exclusive)
         self._increment_version_number_db()
 
@@ -306,27 +329,27 @@ class Node(AbstractNode):
 
     def _get_db_extra(self, key, *args):
         from aiida.backends.djsite.db.models import DbExtra
-        return DbExtra.get_value_for_node(dbnode=self.dbnode,
+        return DbExtra.get_value_for_node(dbnode=self._dbnode,
                                           key=key)
 
     def _del_db_extra(self, key):
         from aiida.backends.djsite.db.models import DbExtra
-        if not DbExtra.has_key(self.dbnode, key):
+        if not DbExtra.has_key(self._dbnode, key):
             raise AttributeError("DbExtra {} does not exist".format(
                 key))
-        return DbExtra.del_value_for_node(self.dbnode, key)
+        return DbExtra.del_value_for_node(self._dbnode, key)
         self._increment_version_number_db()
 
     def _db_iterextras(self):
         from aiida.backends.djsite.db.models import DbExtra
-        extraslist = DbExtra.list_all_node_elements(self.dbnode)
+        extraslist = DbExtra.list_all_node_elements(self._dbnode)
         for e in extraslist:
             yield (e.key, e.getvalue())
 
     def _db_iterattrs(self):
         from aiida.backends.djsite.db.models import DbAttribute
 
-        all_attrs = DbAttribute.get_all_values_for_node(self.dbnode)
+        all_attrs = DbAttribute.get_all_values_for_node(self._dbnode)
         for attr in all_attrs:
             yield (attr, all_attrs[attr])
 
@@ -336,7 +359,7 @@ class Node(AbstractNode):
         # calling iterattrs from here, because iterattrs is slow on each call
         # since it has to call .getvalue(). To improve!
         from aiida.backends.djsite.db.models import DbAttribute
-        attrlist = DbAttribute.list_all_node_elements(self.dbnode)
+        attrlist = DbAttribute.list_all_node_elements(self._dbnode)
         for attr in attrlist:
             yield attr.key
 
@@ -435,13 +458,13 @@ class Node(AbstractNode):
         # otherwise I only get the Django Field F object as a result!
         self._dbnode = DbNode.objects.get(pk=self._dbnode.pk)
 
-    def copy(self):
+    def copy(self, **kwargs):
         newobject = self.__class__()
-        newobject.dbnode.type = self.dbnode.type  # Inherit type
-        newobject.dbnode.label = self.dbnode.label  # Inherit label
+        newobject._dbnode.type = self._dbnode.type  # Inherit type
+        newobject.label = self.label  # Inherit label
         # TODO: add to the description the fact that this was a copy?
-        newobject.dbnode.description = self.dbnode.description  # Inherit description
-        newobject.dbnode.dbcomputer = self.dbnode.dbcomputer  # Inherit computer
+        newobject.description = self.description  # Inherit description
+        newobject._dbnode.dbcomputer = self._dbnode.dbcomputer  # Inherit computer
 
         for k, v in self.iterattrs():
             if k != Sealable.SEALED_KEY:
@@ -454,11 +477,11 @@ class Node(AbstractNode):
 
     @property
     def uuid(self):
-        return unicode(self.dbnode.uuid)
+        return unicode(self._dbnode.uuid)
 
     @property
     def id(self):
-        return self.dbnode.id
+        return self._dbnode.id
 
     @property
     def dbnode(self):
@@ -468,7 +491,7 @@ class Node(AbstractNode):
         #            self._dbnode = DbNode.objects.get(pk=self._dbnode.pk)
         return self._dbnode
 
-    def _db_store_all(self, with_transaction=True):
+    def _db_store_all(self, with_transaction=True, use_cache=None):
         """
         Store the node, together with all input links, if cached, and also the
         linked nodes, if they were not stored yet.
@@ -489,11 +512,13 @@ class Node(AbstractNode):
             # Always without transaction: either it is the context_man here,
             # or it is managed outside
             self._store_input_nodes()
-            self.store(with_transaction=False)
+            self.store(with_transaction=False, use_cache=use_cache)
             self._store_cached_input_links(with_transaction=False)
 
         return self
 
+    def get_user(self):
+        return self._dbnode.user
 
     def _store_cached_input_links(self, with_transaction=True):
         """
@@ -560,6 +585,8 @@ class Node(AbstractNode):
         :parameter with_transaction: if False, no transaction is used. This
           is meant to be used ONLY if the outer calling function has already
           a transaction open!
+
+        :param bool use_cache: Whether I attempt to find an equal node in the DB.
         """
         # TODO: This needs to be generalized, allowing for flexible methods
         # for storing data and its attributes.
@@ -592,7 +619,7 @@ class Node(AbstractNode):
                 self._dbnode.save()
                 # Save its attributes 'manually' without incrementing
                 # the version for each add.
-                DbAttribute.reset_values_for_node(self.dbnode,
+                DbAttribute.reset_values_for_node(self._dbnode,
                                                   attributes=self._attrs_cache,
                                                   with_transaction=False)
                 # This should not be used anymore: I delete it to
@@ -615,5 +642,8 @@ class Node(AbstractNode):
                 self._repository_folder.abspath, move=True, overwrite=True)
             raise
 
-        return self
+        from aiida.backends.djsite.db.models import DbExtra
+        # I store the hash without cleaning and without incrementing the nodeversion number
+        DbExtra.set_value_for_node(self._dbnode, _HASH_EXTRA_KEY, self.get_hash())
 
+        return self
