@@ -25,7 +25,8 @@ class ServerInfo(Resource):
         utils_conf_keys = ('PREFIX', 'PERPAGE_DEFAULT', 'LIMIT_DEFAULT')
         self.utils_confs = {
             k: kwargs[k]
-            for k in utils_conf_keys if k in kwargs
+            for k in utils_conf_keys
+            if k in kwargs
         }
         self.utils = Utils(**self.utils_confs)
 
@@ -103,7 +104,8 @@ class BaseResource(Resource):
         utils_conf_keys = ('PREFIX', 'PERPAGE_DEFAULT', 'LIMIT_DEFAULT')
         self.utils_confs = {
             k: kwargs[k]
-            for k in utils_conf_keys if k in kwargs
+            for k in utils_conf_keys
+            if k in kwargs
         }
         self.utils = Utils(**self.utils_confs)
         self.method_decorators = {'get': kwargs.get('get_decorators', [])}
@@ -204,7 +206,8 @@ class Node(Resource):
         utils_conf_keys = ('PREFIX', 'PERPAGE_DEFAULT', 'LIMIT_DEFAULT')
         self.utils_confs = {
             k: kwargs[k]
-            for k in utils_conf_keys if k in kwargs
+            for k in utils_conf_keys
+            if k in kwargs
         }
         self.utils = Utils(**self.utils_confs)
         self.method_decorators = {'get': kwargs.get('get_decorators', [])}
@@ -258,7 +261,7 @@ class Node(Resource):
             if filters:
                 usr = filters["user"]["=="]
             else:
-                usr = []
+                usr = None
             results = self.trans.get_statistics(usr)
 
         # TODO Might need to be improved
