@@ -39,7 +39,13 @@ class BaseTranslator(object):
 
     _result_type = __label__
 
-    _default = _default_projections = []
+    _default = _default_projections = ["**"]
+
+    _schema_projections = {
+        "column_order": [],
+        "additional_info": {}
+    }
+
     _is_qb_initialized = False
     _is_id_query = None
     _total_count = None
@@ -70,6 +76,7 @@ class BaseTranslator(object):
 
         self._default = Class._default
         self._default_projections = Class._default_projections
+        self._schema_projections = Class._schema_projections
         self._is_qb_initialized = Class._is_qb_initialized
         self._is_id_query = Class._is_id_query
         self._total_count = Class._total_count
@@ -92,20 +99,16 @@ class BaseTranslator(object):
         self.qb = QueryBuilder()
 
         self.LIMIT_DEFAULT = kwargs['LIMIT_DEFAULT']
-
-        if 'custom_schema' in kwargs:
-            self.custom_schema = kwargs['custom_schema']
-        else:
-            self.custom_schema = None
+        self.schema = None
 
     def __repr__(self):
         """
         This function is required for the caching system to be able to compare
-        two NodeTranslator objects. Comparison is done on the value returned by
-        __repr__
+        two NodeTranslator objects. Comparison is done on the value returned by __repr__
+
         :return: representation of NodeTranslator objects. Returns nothing
-        because the inputs of self.get_nodes are sufficient to determine the
-        identity of two queries.
+            because the inputs of self.get_nodes are sufficient to determine the
+            identity of two queries.
         """
         return ""
 
@@ -118,91 +121,45 @@ class BaseTranslator(object):
         orm_class = get_object_from_string(class_string)
 
         # Construct the json object to be returned
-        basic_schema = orm_class.get_db_columns()
+        basic_schema = orm_class.get_schema()
 
-        """
-        Determine the API schema (spartially overlapping with the ORM/database one).
-        When the ORM is based on django, however, attributes and extras are not colums of the database but are
-        nevertheless         valid projections. We add them by hand into the API schema.
-        """
-        # TODO change the get_db_columns method to include also relationships such as attributes, extras, input,
-        # and outputs        in order to have a more complete definition of the schema.
+        schema = {}
+        ordering = []
+
+        # get addional info and column order from translator class
+        # and combine it with basic schema
+        if len(self._schema_projections["column_order"]) > 0:
+            for field in self._schema_projections["column_order"]:
+
+                # basic schema
+                if field in basic_schema.keys():
+                    schema[field] = basic_schema[field]
+                else:
+                    ## Note: if column name starts with user_* get the schema information from
+                    # user class. It is added mainly to handle user_email case.
+                    # TODO need to improve
+                    field_parts = field.split("_")
+                    if field_parts[0] == "user" and field != "user_id" and len(field_parts) > 1:
+                        from aiida.orm.user import User
+                        user_schema = User.get_schema()
+                        if field_parts[1] in user_schema.keys():
+                            schema[field] = user_schema[field_parts[1]]
+                        else:
+                            raise KeyError("{} is not present in user schema".format(field))
+                    else:
+                        raise KeyError("{} is not present in ORM basic schema".format(field))
+
+                # additional info defined in translator class
+                if field in self._schema_projections["additional_info"]:
+                    schema[field].update(self._schema_projections["additional_info"][field])
+                else:
+                    raise KeyError("{} is not present in default projection additional info".format(field))
+
+            # order
+            ordering = self._schema_projections["column_order"]
 
-        if self._default_projections == ['**']:
-            schema = basic_schema  # No custom schema, take the basic one
         else:
-
-            # Non-schema possible projections (only for nodes when django is backend)
-            non_schema_projs = ('attributes', 'extras')
-            # Sub-projections of JSON fields (applies to both SQLA and Django)
-            non_schema_proj_prefix = ('attributes.', 'extras.')
-
-            schema_key = []
-            schema_values = []
-
-            for k in self._default_projections:
-                if k in basic_schema.keys():
-                    schema_key.append(k)
-                    schema_values.append(basic_schema[k])
-                elif k in non_schema_projs:
-                    # Catches 'attributes' and 'extras'
-                    schema_key.append(k)
-                    value = dict(type=dict, is_foreign_key=False)
-                    schema_values.append(value)
-                elif k.startswith(non_schema_proj_prefix):
-                    # Catches 'attributes.<key>' and 'extras.<key>'
-                    schema_key.append(k)
-                    value = dict(type=None, is_foreign_key=False)
-                    schema_values.append(value)
-
-            schema = dict(zip(schema_key, schema_values))
-
-        def table2resource(table_name):
-            """
-            Convert the related_tablevalues to the RESTAPI resources
-            (orm class/db table ==> RESTapi resource)
-
-            :param table_name (str): name of the table (in SQLA is __tablename__)
-            :return: resource_name (str): name of the API resource
-            """
-            # TODO Consider ways to make this function backend independent (one
-            # idea would be to go from table name to aiida class name which is
-            # unique)
-            if BACKEND == BACKEND_DJANGO:
-                (spam, resource_name) = issingular(table_name[2:].lower())
-            elif BACKEND == BACKEND_SQLA:
-                (spam, resource_name) = issingular(table_name[5:])
-            elif BACKEND is None:
-                raise ConfigurationError("settings.BACKEND has not been set.\n"
-                                         "Hint: Have you called "
-                                         "aiida.load_dbenv?")
-            else:
-                raise ConfigurationError("Unknown settings.BACKEND: {}".format(
-                    BACKEND))
-            return resource_name
-
-        for k, v in schema.iteritems():
-
-            # Add custom fields to the column dictionaries
-            if 'fields' in self.custom_schema:
-                if k in self.custom_schema['fields'].keys():
-                    schema[k].update(self.custom_schema['fields'][k])
-
-            # Convert python types values into strings
-            schema[k]['type'] = str(schema[k]['type'])[7:-2]
-
-            # Construct the 'related resource' field from the 'related_table'
-            # field
-            if v['is_foreign_key'] == True:
-                schema[k]['related_resource'] = table2resource(
-                    schema[k].pop('related_table'))
-
-        # TODO Construct the ordering (all these things have to be moved in matcloud_backend)
-        if self._default_projections != ['**']:
-            ordering = self._default_projections
-        else:
-            # random ordering if not set explicitely in
-            ordering = schema.keys()
+            raise ConfigurationError("Define column order to get schema for {}".format(self._aiida_type))
 
         return dict(fields=schema, ordering=ordering)
 
@@ -238,7 +195,8 @@ class BaseTranslator(object):
 
     def get_total_count(self):
         """
-        Returns the number of rows of the query
+        Returns the number of rows of the query.
+
         :return: total_count
         """
         ## Count the results if needed
@@ -252,13 +210,15 @@ class BaseTranslator(object):
         Add filters in query_help.
 
         :param filters: it is a dictionary where keys are the tag names
-         given in the path in query_help and their values are the dictionary
-         of filters want to add for that tag name. Format for the Filters
-         dictionary:
-         filters = {
+            given in the path in query_help and their values are the dictionary
+            of filters want to add for that tag name. Format for the Filters
+            dictionary::
+
+                filters = {
                     "tag1" : {k1:v1, k2:v2},
                     "tag2" : {k1:v1, k2:v2},
-                  }
+                }
+
         :return: query_help dict including filters if any.
         """
         if isinstance(filters, dict):
@@ -360,7 +320,8 @@ class BaseTranslator(object):
         :param filters: dictionary with the filters
         :param orders: dictionary with the order for each tag
         :param orders: dictionary with the projections
-        :param id (integer): id of a specific node
+        :param id: id of a specific node
+        :type id: int
         """
 
         tagged_filters = {}
@@ -455,9 +416,11 @@ class BaseTranslator(object):
 
     def get_formatted_result(self, label):
         """
-        Runs the query and retrieves results tagged as "label"
-        :param label (string): the tag of the results to be extracted out of
-        the query rows.
+        Runs the query and retrieves results tagged as "label".
+
+        :param label: the tag of the results to be extracted out of
+          the query rows.
+        :type label: str
         :return: a list of the query results
         """
 
@@ -479,10 +442,9 @@ class BaseTranslator(object):
 
     def get_results(self):
         """
-        Returns either list of nodes or details of single node from database
+        Returns either list of nodes or details of single node from database.
 
-        :return: either list of nodes or details of single node
-        from database
+        :return: either list of nodes or details of single node from database
         """
 
         ## Check whether the querybuilder object has been initialized
@@ -501,18 +463,17 @@ class BaseTranslator(object):
 
     def _check_id_validity(self, id):
         """
-        Checks whether a id full id or id starting pattern) corresponds to
-         an object of the expected type,
+        Checks whether id corresponds to an object of the expected type,
         whenever type is a valid column of the database (ex. for nodes,
         but not for users)
         
-        :param id: id, or id starting pattern
+        :param id: id (or id starting pattern)
         
-        :return: True if id valid (invalid). If True, sets the
-            id filter attribute correctly
+        :return: True if id valid, False if invalid. If True, sets the id
+          filter attribute correctly
             
-        :raise: RestValidationError if No node is found or id pattern does
-        not identify a unique node
+        :raise RestValidationError: if no node is found or id pattern does
+          not identify a unique node
         """
         from aiida.common.exceptions import MultipleObjectsError, NotExistent
 
