@@ -59,12 +59,48 @@ class NodeTranslator(BaseTranslator):
         # basic initialization
         super(NodeTranslator, self).__init__(Class=Class, **kwargs)
 
-        # Extract the default projections from custom_schema if they are defined
-        if self.custom_schema is not None and 'columns' in self.custom_schema:
-            self._default_projections = self.custom_schema['columns'][
-                self.__label__]
-        else:
-            self._default_projections = ['**']
+        self._default_projections = [
+            "id",
+            "label",
+            "type",
+            "ctime",
+            "mtime",
+            "uuid",
+            "user_id",
+            "user_email",
+            "attributes",
+            "extras"
+        ]
+
+        ## node schema
+        # All the values from column_order must present in additional info dict
+        # Note: final schema will contain details for only the fields present in column order
+        self._schema_projections = {
+            "column_order": [
+                "id",
+                "label",
+                "type",
+                "ctime",
+                "mtime",
+                "uuid",
+                "user_id",
+                "user_email",
+                "attributes",
+                "extras"
+            ],
+            "additional_info": {
+                "id": {"is_display": True},
+                "label": {"is_display": False},
+                "type": {"is_display": True},
+                "ctime": {"is_display": True},
+                "mtime": {"is_display": True},
+                "uuid": {"is_display": False},
+                "user_id": {"is_display": False},
+                "user_email": {"is_display": True},
+                "attributes": {"is_display": False},
+                "extras": {"is_display": False}
+            }
+        }
 
         # Inspect the subclasses of NodeTranslator, to avoid hard-coding
         # (should resemble the following tree)
@@ -87,6 +123,7 @@ class NodeTranslator(BaseTranslator):
         """
         sets one of the mutually exclusive values for self._result_type and
         self._content_type.
+
         :param query_type:(string) the value assigned to either variable.
         """
 
@@ -142,9 +179,8 @@ class NodeTranslator(BaseTranslator):
         :param filters: dictionary with the filters
         :param orders: dictionary with the order for each tag
         :param projections: dictionary with the projection. It is discarded
-        if query_type=='attributes'/'extras'
-        :param query_type: (string) specify the result or the content (
-        "attr")
+            if query_type=='attributes'/'extras'
+        :param query_type: (string) specify the result or the content ("attr")
         :param id: (integer) id of a specific node
         """
 
@@ -279,12 +315,15 @@ class NodeTranslator(BaseTranslator):
 
     def _get_subclasses(self, parent=None, parent_class=None, recursive=True):
         """
-        Import all submodules of the package containing the present class,
-        including subpackages recursively, if specified.
-        :parent: package/class. If package look for the classes in submodules.
-            If class, it first looks for the package where it is contained
-        :parent_class: class of which to look for subclasses
-        :recursive: True/False (go recursively into submodules)
+        Import all submodules of the package containing the present class.
+        
+        Includes subpackages recursively, if specified.
+
+        :param parent: package/class. 
+            If package looks for the classes in submodules.
+            If class, first lookss for the package where it is contained
+        :param parent_class: class of which to look for subclasses
+        :param recursive: True/False (go recursively into submodules)
         """
 
         import pkgutil
@@ -416,7 +455,7 @@ class NodeTranslator(BaseTranslator):
         :returns: list of calc inputls command
         """
 
-        if node.dbnode.type.startswith("calculation"):
+        if node.type.startswith("calculation"):
             from aiida.restapi.translator.calculation import CalculationTranslator
             return CalculationTranslator.get_retrieved_inputs(node, filename=filename, rtype=rtype)
         return []
@@ -432,7 +471,7 @@ class NodeTranslator(BaseTranslator):
         :returns: list of calc outputls command
         """
 
-        if node.dbnode.type.startswith("calculation"):
+        if node.type.startswith("calculation"):
             from aiida.restapi.translator.calculation import CalculationTranslator
             return CalculationTranslator.get_retrieved_outputs(node, filename=filename, rtype=rtype)
         return []
@@ -465,18 +504,19 @@ class NodeTranslator(BaseTranslator):
         Returns either a list of nodes or details of single node from database
 
         :return: either a list of nodes or the details of single node
-        from the database
+            from the database
         """
         if self._content_type is not None:
             return self._get_content()
         else:
             return super(NodeTranslator, self).get_results()
 
-    def get_statistics(self, user_email=None):
-        "Return statistics for a given node"
+    def get_statistics(self, user_pk=None):
+        """Return statistics for a given node"""
+
         from aiida.backends.utils import QueryFactory
         qmanager = QueryFactory()()
-        return qmanager.get_creation_statistics(user_email=user_email)
+        return qmanager.get_creation_statistics(user_pk=user_pk)
 
 
     def get_io_tree(self, uuid_pattern):
@@ -511,11 +551,11 @@ class NodeTranslator(BaseTranslator):
             mainNode = qb.first()[0]
             pk = mainNode.pk
             uuid = mainNode.uuid
-            nodetype = mainNode.dbnode.type
+            nodetype = mainNode.type
             display_type = nodetype.split('.')[-2]
             description = mainNode.get_desc()
             if description == '':
-                description = mainNode.dbnode.type.split('.')[-2]
+                description = mainNode.type.split('.')[-2]
 
             nodes.append({
                 "id": nodeCount,
@@ -542,11 +582,11 @@ class NodeTranslator(BaseTranslator):
                 linktype = input['main--in']['label']
                 pk = node.pk
                 uuid = node.uuid
-                nodetype = node.dbnode.type
+                nodetype = node.type
                 display_type = nodetype.split('.')[-2]
                 description = node.get_desc()
                 if description == '':
-                    description = node.dbnode.type.split('.')[-2]
+                    description = node.type.split('.')[-2]
 
                 nodes.append({
                     "id": nodeCount,
@@ -580,11 +620,11 @@ class NodeTranslator(BaseTranslator):
                 linktype = output['main--out']['label']
                 pk = node.pk
                 uuid = node.uuid
-                nodetype = node.dbnode.type
+                nodetype = node.type
                 display_type = nodetype.split('.')[-2]
                 description = node.get_desc()
                 if description == '':
-                    description = node.dbnode.type.split('.')[-2]
+                    description = node.type.split('.')[-2]
 
                 nodes.append({
                     "id": nodeCount,
@@ -606,4 +646,4 @@ class NodeTranslator(BaseTranslator):
                 })
                 nodeCount += 1
 
-        return {"nodes": nodes, "edges": edges}
\ No newline at end of file
+        return {"nodes": nodes, "edges": edges}
